@@ -406,7 +406,7 @@ if __name__ == '__main__':
 
        # Switching to TLS now
        ctx = SSL.Context(SSL.TLSv1_2_METHOD)
-       ctx.set_cipher_list('RC4,AES')
+       ctx.set_cipher_list(b'RC4,AES')
        tls = SSL.Connection(ctx,s)
        tls.set_connect_state()
        tls.do_handshake()
