@@ -14,7 +14,7 @@
 from struct import unpack
 import socket
 
-from ImpactPacket import Header
+from impacket.ImpactPacket import Header
 from impacket import LOG
 
 IP_ADDRESS_LENGTH = 4
@@ -49,10 +49,8 @@ class CDP(Header):
         # Remove version (1 byte), TTL (1 byte), and checksum (2 bytes)
         buff = aBuffer[4:]
         l = []
-        finish = False
         while buff:
             elem = CDPElementFactory.create(buff)
-            data = elem.get_data()
             l.append( elem )
             buff = buff[ elem.get_length() : ]
         return l
@@ -80,7 +78,6 @@ class CDP(Header):
 
 
     def __str__(self):
-        knowcode = 0
         tmp_str = 'CDP Details:\n'
         for element in self._elements:
             tmp_str += "** Type:" + str(element.get_type()) + " " + str(element) + "\n"
@@ -125,13 +122,19 @@ class CDPElement(Header):
 
     def get_length(self):
         return self.get_word(2)
-                
-    def get_data(self):        
-        return self.get_bytes().tostring()[4:self.get_length()]
+
+    def get_data(self):
+        try:
+            return self.get_bytes().tobytes()[4:self.get_length()]
+        except AttributeError:  # Python < 3.2
+            return self.get_bytes().tostring()[4:self.get_length()]
 
     def get_ip_address(self, offset = 0, ip = None):
         if not ip:
-            ip = self.get_bytes().tostring()[offset : offset + IP_ADDRESS_LENGTH]
+            try:
+                ip = self.get_bytes().tobytes()[offset : offset + IP_ADDRESS_LENGTH]
+            except AttributeError:  # Python < 3.2
+                ip = self.get_bytes().tostring()[offset : offset + IP_ADDRESS_LENGTH]
         return socket.inet_ntoa( ip )
         
 class CDPDevice(CDPElement):
@@ -152,7 +155,10 @@ class Address(CDPElement):
     def __init__(self, aBuffer = None):
         CDPElement.__init__(self, aBuffer)
         if aBuffer:
-            data = self.get_bytes().tostring()[8:]
+            try:
+                data = self.get_bytes().tobytes()[8:]
+            except AttributeError:  # Python < 3.2
+                data = self.get_bytes().tostring()[8:]
             self._generateAddressDetails(data)
 
     def _generateAddressDetails(self, buff):
@@ -356,10 +362,16 @@ class ProtocolHello(CDPElement):
         return self.get_byte(19)
 
     def get_cluster_command_mac(self):
-        return self.get_bytes().tostring()[20:20+6]
+        try:
+            return self.get_bytes().tobytes()[20:20+6]
+        except AttributeError:  # Python < 3.2
+            return self.get_bytes().tostring()[20:20+6]
             
     def get_switch_mac(self):
-        return self.get_bytes().tostring()[28:28+6]
+        try:
+            return self.get_bytes().tobytes()[28:28+6]
+        except AttributeError:  # Python < 3.2
+            return self.get_bytes().tostring()[28:28+6]
             
     def get_management_vlan(self):
         return self.get_word(36)
